package edu.kit.iti.formal.nonogram;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.List;

/**
 * @author Alexander Weigl
 * @version 1 (14.11.17)
 */
public class FastRunner implements Runnable {
    private static boolean NO_CHECK = false;
    private List<Long> timings = new ArrayList<>();
    private volatile long startTime, stopTime;
    private int success;
    private int all;

    public static void main(String[] args) {
        FastRunner runner = new FastRunner();
        NO_CHECK = Boolean.valueOf(
                System.getenv().getOrDefault("NO_CHECK", "false"));
        int time = Integer.valueOf(System.getenv().getOrDefault("TIMEOUT", "100"));

        System.out.println("Solution checking is " + (NO_CHECK ? "disabled" : "enabled") + ".");
        System.out.println("Timeout is  " + time + " seconds.");
        try {
            runner.run();
        } finally {
            Runner.statistics(runner.success, runner.all, runner.startTime, runner.stopTime, runner.timings);
            System.exit(0);
        }
    }

    public boolean checkSolver(String puzzle) throws ClassNotFoundException, NoSuchMethodException, InvocationTargetException, InstantiationException, IllegalAccessException {
        Nonogram monogram = Nonogram.from(puzzle);

        System.out.printf("== %s ======================================================%n", monogram.getID());
        NonogramSolver solver = Runner.solverInstance(monogram);

        long start = System.currentTimeMillis();
        boolean ans = solver.solve();
        timings.add(System.currentTimeMillis() - start);

        boolean satisfiable = !monogram.getID().startsWith("u");

        System.out.println("Riddle is " + (!satisfiable ? "not " : "") + "satisfiable");
        System.out.println("Your answer is " + (ans ? "sat" : "unsat"));

        if (ans && !NO_CHECK) {
            if (! Runner.printAndCheckSolution(monogram, solver.getSolution())) {
                System.out.println("There is an error in your solution!");
                return false;
            }
        }

        if (ans == satisfiable)
            System.out.println("SOLVED!");
        else
            System.out.println("NOT SOLVED :(");

        return ans == satisfiable;
    }

    public void run() {
        try {
            startTime = System.currentTimeMillis();
            success = 0;
            all = 0;
            for (String s : Runner.findRiddles()) {
                all++;
                success += checkSolver(s) ? 1 : 0;
            }
        } catch (ClassNotFoundException e) {
            System.err.println("Could not from the solution by class name.");
            System.err.println("There should be class 'MyNonogramSolver' in the default package on the classpath.");
            e.printStackTrace();
            System.exit(3);
        } catch (NoSuchMethodException | InvocationTargetException | InstantiationException | IllegalAccessException e) {
            System.err.println("Constructor not found");
            //System.err.println("There should be constructor 'public MyKuromasuSolver(Kuromasu)' in your solution class.");
            System.err.println(e.getMessage());
            e.printStackTrace();
            System.exit(4);
        } finally {
            stopTime = System.currentTimeMillis();
        }
    }


}
