package edu.kit.iti.formal.nonogram;

import org.sat4j.core.VecInt;
import org.sat4j.minisat.SolverFactory;
import org.sat4j.specs.ContradictionException;
import org.sat4j.specs.ISolver;
import org.sat4j.specs.IVecInt;

import java.util.List;

/**
 * Base class for nonogram solvers!
 * @author Alexander Weigl
 * @version 1 (14.11.17)
 */
public abstract class NonogramSolver {
    protected ISolver solver = SolverFactory.newDefault();
    protected Nonogram game;
    protected boolean[][] solution;

    public final void init(Nonogram game) {
        this.game = game;
        solution = new boolean[game.getHeight()][game.getWidth()];
// already initialised automatically
//        for (int i = 0; i < solution.length; i++) {
//            for (int j = 0; j < solution[i].length; j++) {
//                solution[i][j] = false;
//            }
//        }
    }

    public abstract boolean solve();

    public boolean[][] getSolution() {
        return solution;
    }

    /**
     * Adds the given literals as a clause to the sat solver.
     * Abbreviation for:
     * <code><pre>
     * IVecInt lits=new VecInt(literals);
     * solver.addClause(lits);
     * </pre></code>
     *
     * @param literals
     * @throws ContradictionException
     */
    public void addClause(int... literals) throws ContradictionException {
        IVecInt lits = new VecInt(literals);
        solver.addClause(lits);
    }

    /**
     * Adds the given literals as a clause to the sat solver.
     *
     * @param literals a non null, non-empty list of literals
     * @throws ContradictionException
     * @see #addClause(int...)
     */
    public void addClause(List<Integer> literals) throws ContradictionException {
        int[] ary = new int[literals.size()];
        for (int i = 0; i < ary.length; i++) {
            ary[i] = literals.get(i);
        }
        addClause(ary);
    }

    /**
     * Prints the current solution. Debugging purposes
     * @see #getSolution()
     */
    public void printSolution() {
        boolean[][] sol = getSolution();
        for (int i = 0; i < sol.length; i++) {
            for (int j = 0; j < sol[i].length; j++) {
                System.out.print(sol[i][j] ? "x" : "-");
            }
            System.out.println();
        }
    }
}
